"""
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
P3D script : generateRALMaterialLibrary_From_2021_1
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Script Target : 
|~----------------------------------------------------------------------------~|

 This script affects : None

|~----------------------------------------------------------------------------~|

Compatible Versions : 
|~----------------------------------------------------------------------------~|

 This script is compatible with versions : 2021.1 and next

|~----------------------------------------------------------------------------~|

Requirements : 
|~----------------------------------------------------------------------------~|

 Special conditions : None

|~----------------------------------------------------------------------------~|

Description :
|~----------------------------------------------------------------------------~|

 This script generates a material library creating variations of temmplate 
 materials using RAL color variations.
 Template materials come from standard Patchwork 3D library : 
  - black high gloss plastic.kmt
  - black  plastic .kmt
  - chrome.kmt
  - aluminium.kmt
  - dark_glass.kmt

|~----------------------------------------------------------------------------~|

Usage:
|~----------------------------------------------------------------------------~|
 
 Classic use case in P3D :  Scripting/Execute Script -> choose script

 Or in command line, with p3dsdk in the Python Path :
  - direct call sample
 python generateRALMaterialLibrary_From_2021_1.py "ral_standard.csv" "F:\kmt2" "F:\library"
  - sample displaying UI
  python generateRALMaterialLibrary_From_2021_1.py

|~----------------------------------------------------------------------------~|

Data output format :
|~----------------------------------------------------------------------------~|

 Creates a "Material" folder at chosen output directory, containing all 
 generated materials

|~----------------------------------------------------------------------------~|

"""


import sys
import argparse
import csv
import os
import typing
from typing import List
from p3dsdk import Material, KMTFile
from UI_RAL_utils import ScriptRalUi, ScriptRalUiError



template_name_to_general_name = {
    "black high gloss plastic" : "high gloss plastic",
    "black  plastic " : "plastic",
    "chrome" : "chrome",
    "aluminium" : "aluminium",
    "dark_glass" : "glass"
}


def list_template_kmt() -> List[str]:
    return [f"{name}.kmt" for name in template_name_to_general_name]


def create_folder(folder_path: str):
    try:
        os.makedirs(folder_path)
    except FileExistsError:
        print("Dossier " + folder_path + " existe déjà")


def parse_color_rgb(string_rgb: str) -> float():
    result = string_rgb.split('-')
    return float(result[0]), float(result[1]), float(result[2])


def create_material(template: Material, color: float()) -> Material:
    material = template.duplicate()
    color = tuple(colorConverted/255 for colorConverted in color)

    if template.name == "black high gloss plastic" or template.name == "black  plastic ":
        material.diffuse_color = color
    elif template.name == "aluminium" or template.name == "chrome":
        material.reflection_color = color
    elif template.name == "dark_glass":
        material.diffuse_filter = color
    return material


def create_kmt(folder_path: str, reader_spam, template_material: Material):
    for row in reader_spam:
        if row[1] != " RGB":
            color = parse_color_rgb(row[1])
            material = create_material(template_material, color)
            material.name = row[0] + " - " + row[4] + " - " + template_name_to_general_name[template_material.name]
            material.export_kmt(os.path.join(folder_path, material.name.rstrip() + ".kmt"))


def generate_libraries(csv_file_args: str, folder_dest: str, list_path_files_kmt):
    create_folder(os.path.join(folder_dest, "Materials"))
    for file_path in list_path_files_kmt:
        try:
            template_material = KMTFile.open(file_path)
        except Exception as e:
            print(e)
        path_folder = os.path.join(folder_dest, "Materials", template_name_to_general_name[template_material.name])
        create_folder(path_folder)

        with open(csv_file_args, newline='', encoding='utf-8') as csv_file:
            spam_reader = csv.reader(csv_file, delimiter=',')
            create_kmt(path_folder, spam_reader, template_material)
        print("Matériaux générés dans le dossier:", path_folder)


def find_template_kmt(folder_kmt: str) -> typing.Tuple[str, str]:
    found_kmt_path = []
    for r, d, f in os.walk(folder_kmt):
        for file in f:
            if file in list_template_kmt():
                found_kmt_path.append(os.path.join(r, file))

    found_kmt_basenames = [os.path.basename(element) for element in found_kmt_path]
    missing_kmt = list(set(list_template_kmt()).difference(found_kmt_basenames))   
    return (found_kmt_path, missing_kmt)


def execute(csv_file: str, lib_kmt: str, dest_folder: str, found_kmt_path: List[str]):
    print("Executing: \'ScriptRal.py {} {} {}\'".format(csv_file, lib_kmt, dest_folder))
    generate_libraries(csv_file, dest_folder, found_kmt_path) 


def display_ui():
    win = ScriptRalUi()
    ok = win.show()
    if ok:
        (found_kmt_path, missing_kmt) = find_template_kmt(win.path_lib_kmt())
        if len(missing_kmt) > 0:
            win_err = ScriptRalUiError()
            win_err.add_text("Il manque ces matériaux: " + ' '.join(missing_kmt))
            ok = win_err.show()
        if ok and len(missing_kmt) < len(template_name_to_general_name):
            execute(win.path_csv_file(), win.path_lib_kmt(), win.path_folder_dest(), found_kmt_path)


def execute_no_ui():
    parser = argparse.ArgumentParser(description='Process to launch.')
    parser.add_argument('csv_file', metavar='[CSV file]', type=str)
    parser.add_argument('lib_kmt', metavar='[Folder kmt]', type=str)
    parser.add_argument('folder_dest', metavar='[Folder destination]', type=str)
    args = parser.parse_args()

    (found_kmt_path, missing_kmt) = find_template_kmt(args.lib_kmt)
    if len(missing_kmt) > 0:
        for kmt in missing_kmt:
            print("Matériaux manquant : ", kmt)
    if len(missing_kmt) < len(template_name_to_general_name):
        execute(args.csv_file, args.lib_kmt, args.folder_dest, found_kmt_path)


if __name__ == "__main__":
    if len(sys.argv) == 4:
        execute_no_ui()
    else:
        display_ui()
