"""
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
P3D script : generateRALMaterialLibrary_From_2021_1
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Script Target : 
|~----------------------------------------------------------------------------~|

 This script affects : None

|~----------------------------------------------------------------------------~|

Compatible Versions : 
|~----------------------------------------------------------------------------~|

 This script is compatible with versions : 2021.1 and next

|~----------------------------------------------------------------------------~|

Requirements : 
|~----------------------------------------------------------------------------~|

 Special conditions : None

|~----------------------------------------------------------------------------~|

Description :
|~----------------------------------------------------------------------------~|

 This script generates a material library creating variations of temmplate 
 materials using RAL color variations.
 Template materials come from standard Patchwork 3D library : 
  - black high gloss plastic.kmt
  - black  plastic .kmt
  - chrome.kmt
  - aluminium.kmt
  - dark_glass.kmt

|~----------------------------------------------------------------------------~|

Usage:
|~----------------------------------------------------------------------------~|
 
 Classic use case in P3D :  Scripting/Execute Script -> choose script

|~----------------------------------------------------------------------------~|

Data output format :
|~----------------------------------------------------------------------------~|

 Creates material groups in database local library, containing all 
 generated materials

|~----------------------------------------------------------------------------~|

"""


import sys
import argparse
import csv
import os
import typing
from typing import List
from p3dsdk import *
from UI_RAL_utils import ScriptRalUiNoOutput, ScriptRalUiError



template_name_to_general_name = {
    "black high gloss plastic" : "high gloss plastic",
    "black  plastic " : "plastic",
    "chrome" : "chrome",
    "aluminium" : "aluminium",
    "dark_glass" : "glass"
}


def list_template_kmt() -> List[str]:
    return [f"{name}.kmt" for name in template_name_to_general_name]

def parse_color_rgb(string_rgb: str) -> float():
    result = string_rgb.split('-')
    return float(result[0]), float(result[1]), float(result[2])


def create_material(template: Material, color: float()) -> Material:
    material = template.duplicate()
    color = tuple(colorConverted/255 for colorConverted in color)

    if template.name == "black high gloss plastic" or template.name == "black  plastic ":
        material.diffuse_color = color
    elif template.name == "aluminium" or template.name == "chrome":
        material.reflection_color = color
    elif template.name == "dark_glass":
        material.diffuse_filter = color
    return material


def create_kmt(material_group: MaterialGroup, reader_spam, template_material: Material):
    for row in reader_spam:
        if row[1] != " RGB":
            color = parse_color_rgb(row[1])
            material = create_material(template_material, color)
            material.name = row[0] + " - " + row[4] + " - " + template_name_to_general_name[template_material.name]


def generate_libraries(csv_file_args: str, list_path_files_kmt):
    with CurrentP3DSession(fallback_port=33900) as p3d:
        material_groups = p3d.data.list_material_groups()
        for file_path in list_path_files_kmt:
            
            try:
                template_material = p3d.data.active_material_group.import_kmt(file_path)
            except Exception as e:
                print(e)

            material_group_name = template_name_to_general_name[template_material.name]
            found_group = [x for x in material_groups if x.name == material_group_name]
            material_group = found_group[0] if found_group else p3d.data.create_material_group(material_group_name)
            
            template_material.group = material_group

            with open(csv_file_args, newline='', encoding='utf-8') as csv_file:
                spam_reader = csv.reader(csv_file, delimiter=',')
                create_kmt(material_group, spam_reader, template_material)

            template_material.remove()
            print("Materials generated in group: ", material_group_name)


def execute(csv_file: str, lib_kmt: str, found_kmt_paths: List[str]):
    print("Executing: \'ScriptRal.py {} {}\'".format(csv_file, lib_kmt))
    generate_libraries(csv_file, found_kmt_paths) 


def find_template_kmt(folder_kmt: str) -> typing.Tuple[str, str]:
    found_kmt_paths = []
    for r, d, f in os.walk(folder_kmt):
        for file in f:
            if file in list_template_kmt():
                found_kmt_paths.append(os.path.join(r, file))

    found_kmt_basenames = [os.path.basename(element) for element in found_kmt_paths]
    missing_kmt = list(set(list_template_kmt()).difference(found_kmt_basenames))   
    return (found_kmt_paths, missing_kmt)


def display_ui():
    win = ScriptRalUiNoOutput()
    ok = win.show()
    if ok:
        (found_kmt_paths, missing_kmt) = find_template_kmt(win.path_lib_kmt())
        if len(missing_kmt) > 0:
            win_err = ScriptRalUiError()
            win_err.add_text("Il manque ces matériaux: " + ' '.join(missing_kmt))
            ok = win_err.show()
        if ok and len(missing_kmt) < len(template_name_to_general_name):
            execute(win.path_csv_file(), win.path_lib_kmt(), found_kmt_paths)


if __name__ == "__main__":
    display_ui()
